window > stack {
    background-color: @theme_bg_color;
    box-shadow: none;
}

window > stack:backdrop {
    background-color: @theme_unfocused_bg_color;
    box-shadow: none;
}

/* world */

.world-tile image,
.world-tile image:backdrop {
    border: 2px solid rgb(141, 141, 141);
}

.world-tile .stripe,
.world-tile .stripe:backdrop,
.tile.prelight .world-tile .stripe,
.tile.prelight .world-tile .stripe:backdrop {
    color: black;
    text-shadow: none;
    background-color: white;
}

.night .world-tile .stripe,
.night .world-tile .stripe:backdrop,
.tile.prelight .night .world-tile .stripe,
.tile.prelight .night .world-tile .stripe:backdrop {
    color: white;
    text-shadow: none;
    background-color: black;
}

/* alarms */

.alarm-tile,
.alarm-tile:backdrop,
.tile.prelight .alarm-tile,
.tile.prelight .alarm-tile:backdrop {
    color: @theme_fg_color;
    text-shadow: none;
    background-image: none;
    background-color: @theme_bg_color;
    border: 2px solid rgb(141, 141, 141);
}

.active .alarm-tile,
.active .alarm-tile:backdrop,
.tile.prelight .active .alarm-tile,
.tile.prelight .active .alarm-tile:backdrop {
    color: @theme_selected_fg_color;
    background-image: none;
    background-color: @theme_selected_bg_color;
}

.snoozing .alarm-tile,
.snoozing .alarm-tile:backdrop,
.tile.prelight .snoozing .alarm-tile,
.tile.prelight .snoozing .alarm-tile:backdrop {
    color: @theme_fg_color;
    text-shadow: none;
    background-image: none;
    background-color: @warning_color;
}
